#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-07-01
#
# extended if64 to use ifNAme instead of ifDescr
#
# 2021-0929: fixed empty interface name handling
#

from typing import (
    List,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    type_defs,
)
from cmk.base.plugins.agent_based.utils import if64, interfaces

_interface_displayhints = {
    'ethernet': 'eth',
    'fastethernet': 'Fa',
    'gigabitethernet': 'Gi',
    'tengigabitethernet': 'Te',
    'twentyfivegige': 'Tw',
    'fortygigabitethernet': 'Fo',
    'hundredgigabitethernet': 'Hu',
    'port-channel': 'Po',
    'tunnel': 'Tu',
    'loopback': 'Lo',
    'cellular': 'Cel',
    'vlan': 'Vlan',
    'management': 'Ma',
}


def _get_short_if_name(ifname: str) -> str:
    """
    returns short interface name from long interface name
    ifname: is the long interface name
    :type ifname: str
    """

    for ifname_prefix in _interface_displayhints.keys():
        if ifname.lower().startswith(ifname_prefix.lower()):
            ifname_short = _interface_displayhints[ifname_prefix]
            return ifname.lower().replace(ifname_prefix.lower(), ifname_short, 1)
    return ifname


def parse_if64name(string_table: List[type_defs.StringByteTable]) -> interfaces.Section:
    for interface in string_table[0]:
        if interface[-1] != '':
            interface[1] = interface[-1]  # replace ifDescr with ifName (.1.3.6.1.2.1.31.1.1.1.1)
        interface[1] = _get_short_if_name(interface[1])  # ifDescr (.1.3.6.1.2.1.2.2.1.2)
        interface.pop()  # remove last entry from interfaces (ifName)  -> return original if64 format
    return if64.parse_if64(string_table)


register.snmp_section(
    name='if64name',
    parsed_section_name='if64',
    parse_function=parse_if64name,
    fetch=[
        SNMPTree(
            base=if64.BASE_OID,
            oids=if64.END_OIDS + ['31.1.1.1.1']  # IF-MIB::ifName,
        ),
    ],
    detect=if64.HAS_ifHCInOctets,
    supersedes=['if64', 'interfaces', 'if', 'statgrab_net'],
)
